#include <stdio.h>
#include <cuda_runtime.h>


__global__ void naive_histo ( int *d_bins, const int *d_in, const int BIN_COUNT ) {
    int idx  = threadIdx.x + blockDim.x * blockIdx.x;
    int item = d_in[idx];
    int bin  = item % BIN_COUNT;
    d_bins[bin]++;
}

__global__ void simple_histo ( int *d_bins, const int *d_in, const int BIN_COUNT ) {
    int idx  = threadidx.x + blockdim.x * blockidx.x;
    int item = d_in[idx]; 
    int bin  = item % BIN_COUNT;
    atomicadd( & (d_bins[bin] ), 1);
}


int main(int argc, char **argv) {
    const int ARRAY_SIZE  = 1024;
    const int ARRAY_BYTES = ARRAY_SIZE * sizeof(int);
    const int BIN_COUNT   = 10;
    const int BIN_BYTES   = BIN_COUNT * sizeof(int);

    srand( 1234ULL );
    // generate the input array on the host
    int h_in[ARRAY_SIZE];
    for ( int i = 0; i < ARRAY_SIZE; i++ ) {
        // put a random number in range [0, BIN_COUNT-1]
        h_in[i] = (int)(rand() % BIN_COUNT);
    }
    int h_bins[BIN_COUNT];
    for ( int i = 0; i < BIN_COUNT; i++ ) {
        h_bins[i] = 0;
    }

    // declare GPU memory pointers
    int * d_in;
    int * d_bins;

    // allocate GPU memory
    cudaMalloc( (void **) &d_in, ARRAY_BYTES);
    cudaMalloc( (void **) &d_bins, BIN_BYTES);

    // transfer the arrays to the GPU
    cudaMemcpy( d_in, h_in, ARRAY_BYTES, cudaMemcpyHostToDevice ); 
    cudaMemcpy( d_bins, h_bins, BIN_BYTES, cudaMemcpyHostToDevice ); 

    int whichKernel = 0;
    if (argc == 2) {
        whichKernel = atoi( argv[1] );
    }
        
    // launch the kernel
    switch(whichKernel) {
    case 0:
        printf("Running naive histo\n");
        naive_histo<<<ARRAY_SIZE / 64, 64>>>( d_bins, d_in, BIN_COUNT );
        break;
    case 1:
        printf("Running simple histo\n");
        simple_histo<<<ARRAY_SIZE / 64, 64>>>( d _bins, d_in, BIN_COUNT );
        break;
    default:
        fprintf( stderr, "error: ran no kernel\n" );
        exit( EXIT_FAILURE );
    }

    // copy back the sum from GPU
    cudaMemcpy(h_bins, d_bins, BIN_BYTES, cudaMemcpyDeviceToHost);

    for ( int i = 0; i < BIN_COUNT; i++ ) {
        printf("bin %d: count %d\n", i, h_bins[i]);
    }

    // free GPU memory allocation
    cudaFree(d_in);
    cudaFree(d_bins);
        
    return 0;
}