#include <stdio.h>
#include <cuda.h>
#include <cuda_runtime.h>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/opencv.hpp>

#include "utils.h"
#include "timer.h"

cv::Mat inputImageRGBA;
cv::Mat outputImageRGBA;

int imWidth  = 0;
int imHeight = 0;

void LoadImageToHostMemory ( const std::string filename, uchar4** h_inputImageRGBA, uchar4** h_outputImageRGBA ) { 

  cv::Mat image = cv::imread( filename.c_str(), CV_LOAD_IMAGE_COLOR );
  if ( image.empty() ) {
    std::cerr << "Couldn't open file: " << filename << std::endl;
    exit(1);
  }

  cv::cvtColor( image, inputImageRGBA, CV_BGR2RGBA );

  //allocate memory for the output
  outputImageRGBA.create(image.rows, image.cols, CV_8UC4);

  *h_inputImageRGBA  = (uchar4 *)inputImageRGBA.ptr<unsigned char>(0);
  *h_outputImageRGBA = (uchar4 *)outputImageRGBA.ptr<unsigned char>(0);

  imWidth  = image.cols;
  imHeight = image.rows;
}

void BoxBlurAOS ( uchar4 * d_inputImageRGBA, uchar4 * d_outputImageRGBA, const int imWidth, const int imHeight );
void BoxBlurSOA ( uchar4 * d_inputImageRGBA, uchar4 * d_outputImageRGBA, const int imWidth, const int imHeight );

int main ( int argc, char **argv ) {
  uchar4 *h_inputImageRGBA,  *d_inputImageRGBA;
  uchar4 *h_outputImageRGBA, *d_outputImageRGBA;
  uchar *d_dummy;

  LoadImageToHostMemory ("cinque_terre_small.jpg", &h_inputImageRGBA, &h_outputImageRGBA );

  int numPixels = imWidth * imHeight;

  checkCudaErrors( cudaMalloc( &d_inputImageRGBA, sizeof(uchar4) * numPixels ) ); 
  checkCudaErrors( cudaMalloc( &d_outputImageRGBA, sizeof(uchar4) * numPixels ) ); 
  checkCudaErrors( cudaMalloc( &d_dummy, sizeof(uchar) * numPixels ) ); 
  checkCudaErrors( cudaMemset( d_outputImageRGBA, 0, sizeof(uchar4) * numPixels ) );

  checkCudaErrors( cudaMemcpy( d_inputImageRGBA, h_inputImageRGBA, sizeof(uchar4) * numPixels, cudaMemcpyHostToDevice ) ); // put dummy instead the d_inputImageRGBA to see how checkCudaErrors works

  BoxBlurAOS( d_inputImageRGBA, d_outputImageRGBA, imWidth, imHeight );

  BoxBlurSOA( d_inputImageRGBA, d_outputImageRGBA, imWidth, imHeight );

  checkCudaErrors( cudaMemcpy( h_outputImageRGBA, d_outputImageRGBA, sizeof(uchar4) * numPixels, cudaMemcpyDeviceToHost ) );

  // cv::Mat output(imHeight, imWidth, CV_8UC4, (void*)h_outputImageRGBA );

  // cv::Mat imageOutputBGR;
  // cv::cvtColor(output, imageOutputBGR, CV_RGBA2BGR);

  // //output the image
  // cv::namedWindow( "Display window", cv::WINDOW_AUTOSIZE );// Create a window for display.
  // cv::imshow( "Display window", imageOutputBGR );
  // cv::waitKey( 0 );

  checkCudaErrors( cudaFree( d_inputImageRGBA ) );
  checkCudaErrors( cudaFree( d_outputImageRGBA ) );
  checkCudaErrors( cudaFree( d_dummy ) );

  return 0;
}