#include <stdio.h>
#include <curand.h>
#include <curand_kernel.h>

__host__ void PrintArray ( float * a, const int size ) {
   printf("{ ");
   for ( int i = 0 ; i < size ; i++)  { 
   		printf("%2f ", a[ i ] ); 
   }
   printf("}\n");
}

__global__ void Initialize(curandState *state) {
   int idx = blockIdx.x * blockDim.x + threadIdx.x;
   curand_init( 1234ULL, idx, 0, &state[idx] );
}

__global__ void CreateRand(curandState *state, float *d_array)  {
	int idx      = blockIdx.x * blockDim.x + threadIdx.x;
	d_array[idx] = curand_uniform( &state[idx] );
}

int main( void ) {
	int N_THREADS   = 64;
	int N_BLOCKS    = 2;
	int ARRAY_SIZE  = N_THREADS * N_BLOCKS;
	int ARRAY_BYTES = ARRAY_SIZE * sizeof(float);

	float h_array[ ARRAY_SIZE ];
	float *d_array;
	curandState *d_state;
	
	cudaMalloc( (void**)&d_state, N_THREADS * N_BLOCKS );
	cudaMalloc( (void**)&d_array, ARRAY_BYTES );

	Initialize<<<N_BLOCKS, N_THREADS>>>(d_state);
	CreateRand<<<N_BLOCKS, N_THREADS>>>(d_state, d_array);

	cudaMemcpy ( h_array, d_array, ARRAY_BYTES, cudaMemcpyDeviceToHost );

	PrintArray( h_array, ARRAY_SIZE );

	cudaFree( d_state );	
	cudaFree( d_array );	

	return 0;
}