#include <stdio.h>
#include <math.h>

__host__ void PrintArray ( int * a, const int size ) {
   printf("{ ");
   for ( int i = 0 ; i < size ; i++)  { 
   		printf("%2d ", a[ i ] ); 
   }
   printf("}\n");
}

__global__ void syncExample( int * d_in, int * d_out, int size ) {
   int idx = blockIdx.x * blockDim.x + threadIdx.x;
   extern __shared__ int sh_array[];

   // read in from array
   sh_array[idx] = d_in[idx] ;

   // barrier
   	__syncthreads();

   //do a shift and modify operation
   if ( idx < size - 1  ) {
      int temp = sh_array[ idx + 1];
      __syncthreads();
      sh_array[ idx ] = temp * 2;
   }

   d_out[ idx ] = sh_array[idx];
}

int main ( void ) {
	int ARRAY_SIZE  = 2048;
	int ARRAY_BYTES = ARRAY_SIZE * sizeof( int );
	int h_in[ ARRAY_SIZE ];
	int h_out[ ARRAY_SIZE ];
	int *d_in, *d_out;


	for ( int  i = 0; i < ARRAY_SIZE; ++i ) {
		h_in[ i ]  = 1;
		h_out[ i ] = 0;
	}

	cudaMalloc( (void**)&d_in, ARRAY_BYTES );
	cudaMalloc( (void**)&d_out, ARRAY_BYTES );

	cudaMemcpy( d_in, h_in, ARRAY_BYTES, cudaMemcpyHostToDevice );

	syncExample<<<2, ARRAY_SIZE / 2, ARRAY_BYTES>>>( d_in, d_out, ARRAY_SIZE );

	cudaMemcpy( h_out, d_out, ARRAY_BYTES, cudaMemcpyDeviceToHost );

	PrintArray( h_out, ARRAY_SIZE );

	cudaFree( d_in );
	cudaFree( d_out );	

	return 0;
}